@@ -1,23 +1,12 @@
-/********************************************************************
- KWin - the KDE window manager
- This file is part of the KDE project.
+/*
+    KWin - the KDE window manager
+    This file is part of the KDE project.
 
-Copyright (C) 2006-2007 Rivo Laks <rivolaks@hot.ee>
-Copyright (C) 2010, 2011 Martin Gräßlin <mgraesslin@kde.org>
+    SPDX-FileCopyrightText: 2006-2007 Rivo Laks <rivolaks@hot.ee>
+    SPDX-FileCopyrightText: 2010, 2011 Martin Gräßlin <mgraesslin@kde.org>
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #ifndef KWIN_GLUTILS_H
 #define KWIN_GLUTILS_H
@@ -52,7 +41,7 @@ class GLVertexBufferPrivate;
 //  well as checking for GL version and extensions
 //  Note that GL context has to be created by the time this function is called
 typedef void (*resolveFuncPtr)();
-void KWINGLUTILS_EXPORT initGL(std::function<resolveFuncPtr(const char*)> resolveFunction);
+void KWINGLUTILS_EXPORT initGL(const std::function<resolveFuncPtr(const char*)> &resolveFunction);
 // Cleans up all resources hold by the GL Context
 void KWINGLUTILS_EXPORT cleanupGL();
 
@@ -109,7 +98,7 @@ public:
     /**
      * @return The value of the uniform as a matrix
      * @since 4.7
-     **/
+     */
     QMatrix4x4 getUniformMatrix4x4(const char* name);
 
     enum MatrixUniform {
@@ -129,6 +118,7 @@ public:
 
     enum Vec4Uniform {
         ModulationConstant,
+        TextureClamp,
         Vec4UniformCount
     };
 
@@ -186,6 +176,7 @@ enum class ShaderTrait {
     UniformColor     = (1 << 1),
     Modulate         = (1 << 2),
     AdjustSaturation = (1 << 3),
+    ClampTexture     = (1 << 4),
 };
 
 Q_DECLARE_FLAGS(ShaderTraits, ShaderTrait)
@@ -201,7 +192,7 @@ Q_DECLARE_FLAGS(ShaderTraits, ShaderTrait)
  *
  * @author Martin Gräßlin <mgraesslin@kde.org>
  * @since 4.7
- **/
+ */
 class KWINGLUTILS_EXPORT ShaderManager
 {
 public:
@@ -212,20 +203,13 @@ public:
 
     /**
      * @return The currently bound shader or @c null if no shader is bound.
-     **/
+     */
     GLShader *getBoundShader() const;
 
     /**
      * @return @c true if a shader is bound, @c false otherwise
-     **/
+     */
     bool isShaderBound() const;
-    /**
-     * Is @c true if the environment variable KWIN_GL_DEBUG is set to 1.
-     * In that case shaders are compiled with KWIN_SHADER_DEBUG defined.
-     * @returns @c true if shaders are compiled with debug information
-     * @since 4.8
-     **/
-    bool isShaderDebug() const;
 
     /**
      * Pushes the current shader onto the stack and binds a shader
@@ -235,11 +219,11 @@ public:
 
     /**
      * Binds the @p shader.
-     * To unbind the shader use @link popShader. A previous bound shader will be rebound.
+     * To unbind the shader use popShader. A previous bound shader will be rebound.
      * To bind a built-in shader use the more specific method.
      * @param shader The shader to be bound
      * @see popShader
-     **/
+     */
     void pushShader(GLShader *shader);
 
     /**
@@ -248,7 +232,7 @@ public:
      * It is not safe to call this method if there is no bound shader.
      * @see pushShader
      * @see getBoundShader
-     **/
+     */
     void popShader();
 
     /**
@@ -257,7 +241,7 @@ public:
      * @param vertexSource The source code of the vertex shader
      * @param fragmentSource The source code of the fragment shader.
      * @return The created shader
-     **/
+     */
     GLShader *loadShaderFromCode(const QByteArray &vertexSource, const QByteArray &fragmentSource);
 
     /**
@@ -271,18 +255,18 @@ public:
      * If neither are provided a new shader following the @p traits is generated.
      *
      * @param traits The shader traits for generating the shader
-     * @param vertesSource optional vertex shader source code to be used instead of shader traits
+     * @param vertexSource optional vertex shader source code to be used instead of shader traits
      * @param fragmentSource optional fragment shader source code to be used instead of shader traits
      * @return new generated shader
      * @since 5.6
-     **/
+     */
     GLShader *generateCustomShader(ShaderTraits traits, const QByteArray &vertexSource = QByteArray(), const QByteArray &fragmentSource = QByteArray());
 
     /**
      * Creates a custom shader with the given @p traits and custom @p vertexFile and or @p fragmentFile.
      * The file names specified in @p vertexFile and @p fragmentFile are relative paths to the shaders
      * resource file shipped together with KWin. This means this method can only be used for built-in
-     * effects, for 3rd party effects @link {generateCustomShader} should be used.
+     * effects, for 3rd party effects generateCustomShader should be used.
      *
      * If the @p vertexFile is empty a vertex shader with the given @p traits is generated.
      * If it is not empty the @p vertexFile is used as the source for the vertex shader.
@@ -298,7 +282,7 @@ public:
      * @return new generated shader
      * @see generateCustomShader
      * @since 5.6
-     **/
+     */
     GLShader *generateShaderFromResources(ShaderTraits traits, const QString &vertexFile = QString(), const QString &fragmentFile = QString());
 
     /**
@@ -309,12 +293,12 @@ public:
 
     /**
      * @return a pointer to the ShaderManager instance
-     **/
+     */
     static ShaderManager *instance();
 
     /**
      * @internal
-     **/
+     */
     static void cleanup();
 
 private:
@@ -330,7 +314,6 @@ private:
 
     QStack<GLShader*> m_boundShaders;
     QHash<ShaderTraits, GLShader *> m_shaderHash;
-    bool m_debug;
     QString m_resourcePath;
     static ShaderManager *s_shaderManager;
 };
@@ -350,7 +333,7 @@ private:
  * @endcode
  *
  * @since 4.10
- **/
+ */
 class KWINGLUTILS_EXPORT ShaderBinder
 {
 public:
@@ -359,7 +342,7 @@ public:
      *
      * @param shader The Shader to push on the stack
      * @see ShaderManager::pushShader
-     **/
+     */
     explicit ShaderBinder(GLShader *shader);
     /**
      * @brief Pushes the Shader with the given @p traits to the ShaderManager's stack.
@@ -367,13 +350,13 @@ public:
      * @param traits The traits describing the shader
      * @see ShaderManager::pushShader
      * @since 5.6
-     **/
+     */
     explicit ShaderBinder(ShaderTraits traits);
     ~ShaderBinder();
 
     /**
      * @return The Shader pushed to the Stack.
-     **/
+     */
     GLShader *shader();
 
 private:
@@ -413,14 +396,20 @@ GLShader* ShaderBinder::shader()
  *  later be used to e.g. do post-processing of the scene.
  *
  * @author Rivo Laks <rivolaks@hot.ee>
- **/
+ */
 class KWINGLUTILS_EXPORT GLRenderTarget
 {
 public:
+    /**
+     * Constructs a GLRenderTarget
+     * @since 5.13
+     */
+    explicit GLRenderTarget();
+
     /**
      * Constructs a GLRenderTarget
      * @param color texture where the scene will be rendered onto
-     **/
+     */
     explicit GLRenderTarget(const GLTexture& color);
     ~GLRenderTarget();
 
@@ -428,29 +417,47 @@ public:
      * Enables this render target.
      * All OpenGL commands from now on affect this render target until the
      *  @ref disable method is called
-     **/
+     */
     bool enable();
     /**
      * Disables this render target, activating whichever target was active
      *  when @ref enable was called.
-     **/
+     */
     bool disable();
 
     /**
      * Sets the target texture
      * @param target texture where the scene will be rendered on
      * @since 4.8
-     **/
+     */
     void attachTexture(const GLTexture& target);
 
+    /**
+     * Detaches the texture that is currently attached to this framebuffer object.
+     * @since 5.13
+     */
+    void detachTexture();
+
     bool valid() const  {
         return mValid;
     }
 
+    void setTextureDirty() {
+        mTexture.setDirty();
+    }
+
     static void initStatic();
     static bool supported()  {
         return sSupported;
     }
+
+    /**
+     * Pushes the render target stack of the input parameter in reverse order.
+     * @param targets The stack of GLRenderTargets
+     * @since 5.13
+     */
+    static void pushRenderTargets(QStack <GLRenderTarget*> targets);
+
     static void pushRenderTarget(GLRenderTarget *target);
     static GLRenderTarget *popRenderTarget();
     static bool isRenderTargetBound();
@@ -460,26 +467,26 @@ public:
      *
      * @returns whether framebuffer blitting is supported.
      * @since 4.8
-     **/
+     */
     static bool blitSupported();
 
     /**
      * Blits the content of the current draw framebuffer into the texture attached to this FBO.
      *
-     * Be aware that framebuffer blitting may not be supported on all hardware. Use @link blitSupported to check whether
+     * Be aware that framebuffer blitting may not be supported on all hardware. Use blitSupported to check whether
      * it is supported.
      * @param source Geometry in screen coordinates which should be blitted, if not specified complete framebuffer is used
      * @param destination Geometry in attached texture, if not specified complete texture is used as destination
      * @param filter The filter to use if blitted content needs to be scaled.
      * @see blitSupported
      * @since 4.8
-     **/
+     */
     void blitFromFramebuffer(const QRect &source = QRect(), const QRect &destination = QRect(), GLenum filter = GL_LINEAR);
 
     /**
      * Sets the virtual screen size to @p s.
      * @since 5.2
-     **/
+     */
     static void setVirtualScreenSize(const QSize &s) {
         s_virtualScreenSize = s;
     }
@@ -490,7 +497,7 @@ public:
      * in the virtual geometry space the rendering geometries use.
      * @see virtualScreenGeometry
      * @since 5.9
-     **/
+     */
     static void setVirtualScreenGeometry(const QRect &g) {
         s_virtualScreenGeometry = g;
     }
@@ -500,7 +507,7 @@ public:
      * in the virtual geometry space the rendering system uses.
      * @see setVirtualScreenGeometry
      * @since 5.9
-     **/
+     */
     static QRect virtualScreenGeometry() {
         return s_virtualScreenGeometry;
     }
@@ -520,6 +527,15 @@ public:
         return s_virtualScreenScale;
     }
 
+    /**
+     * The framebuffer of KWin's OpenGL window or other object currently being rendered to
+     *
+     * @since 5.18
+     */
+    static void setKWinFramebuffer(GLuint fb) {
+        s_kwinFramebuffer = fb;
+    }
+
 
 protected:
     void initFBO();
@@ -535,6 +551,7 @@ private:
     static QRect s_virtualScreenGeometry;
     static qreal s_virtualScreenScale;
     static GLint s_virtualScreenViewport[4];
+    static GLuint s_kwinFramebuffer;
 
     GLTexture mTexture;
     bool mValid;
@@ -545,7 +562,7 @@ private:
 enum VertexAttributeType {
     VA_Position = 0,
     VA_TexCoord = 1,
-    VertexAttributeCount = 2
+    VertexAttributeCount = 2,
 };
 
 /**
@@ -704,14 +721,14 @@ public:
      * @see setUseColor
      * @see isUseColor
      * @since 4.7
-     **/
+     */
     void setColor(const QColor& color, bool enableColor = true);
     /**
      * @return @c true if geometry will be painted with a color, @c false otherwise
      * @see setUseColor
      * @see setColor
      * @since 4.7
-     **/
+     */
     bool isUseColor() const;
     /**
      * Enables/Disables rendering the geometry with a color.
@@ -720,14 +737,14 @@ public:
      * @see isUseColor
      * @see setColor
      * @since 4.7
-     **/
+     */
     void setUseColor(bool enable);
 
     /**
      * Resets the instance to default values.
      * Useful for shared buffers.
      * @since 4.7
-     **/
+     */
     void reset();
 
     /**
@@ -762,7 +779,7 @@ public:
     /**
      * @return A shared VBO for streaming data
      * @since 4.7
-     **/
+     */
     static GLVertexBuffer *streamingBuffer();
 
     /**
@@ -770,14 +787,26 @@ public:
      * This is the geometry of the OpenGL window currently being rendered to
      * in the virtual geometry space the rendering geometries use.
      * @since 5.9
-     **/
+     */
     static void setVirtualScreenGeometry(const QRect &g) {
         s_virtualScreenGeometry = g;
     }
 
+    /**
+     * The scale of the OpenGL window currently being rendered to
+     *
+     * @returns the ratio between the virtual geometry space the rendering
+     * system uses and the target
+     * @since 5.11.3
+     */
+    static void setVirtualScreenScale(qreal s) {
+        s_virtualScreenScale = s;
+    }
+
 private:
     GLVertexBufferPrivate* const d;
     static QRect s_virtualScreenGeometry;
+    static qreal s_virtualScreenScale;
 };
 
 } // namespace
